class ContentController < ActionController::Base; end

module Admin
  class AccountsController < ActionController::Base; end
  class PostsController < ActionController::Base; end
  class StuffController < ActionController::Base; end
  class UserController < ActionController::Base; end
  class UsersController < ActionController::Base; end
end

module Api
  class UsersController < ActionController::Base; end
  class ProductsController < ActionController::Base; end
end

class AccountController < ActionController::Base; end
class ArchiveController < ActionController::Base; end
class ArticlesController < ActionController::Base; end
class BarController < ActionController::Base; end
class BlogController < ActionController::Base; end
class BooksController < ActionController::Base; end
class CarsController < ActionController::Base; end
class CcController < ActionController::Base; end
class CController < ActionController::Base; end
class FooController < ActionController::Base; end
class GeocodeController < ActionController::Base; end
class NewsController < ActionController::Base; end
class NotesController < ActionController::Base; end
class PagesController < ActionController::Base; end
class PeopleController < ActionController::Base; end
class PostsController < ActionController::Base; end
class SubpathBooksController < ActionController::Base; end
class SymbolsController < ActionController::Base; end
class UserController < ActionController::Base; end
class UsersController < ActionController::Base; end
